-- Der Assist Helper
--  Provides assistance in setting changing a macro for the
--  main assist.
--
-- By doktorstick (Ummon of Blackhand)
-- 03 January 2007
--
-- $Id: $

-- Core module.
DAH = LibStub ("AceAddon-3.0"):NewAddon ("DerAssistHelper", "AceConsole-3.0")

-- Localization.
local L = LibStub ("AceLocale-3.0"):GetLocale ("DerAssistHelper", true)

-- Option interface for Ace.
local options = {
   name = "DerAssistHelper",
   handler = DAH,
   type = 'group',
   args = {
      assist = {
         type = 'keybinding',
         name = L["Assist Keybind"],
         desc = L["The keybind (e.g., type 'F') to assist."],
         get = "GetAssistOption",
         set = "SetAssistOption",
      },
      setassist = {
         type = 'keybinding',
         name = L["Set Assist Keybind"],
         desc = L["The keybind (e.g., type 'SHIFT-F') to set the assist."],
         get = "GetDefineAssistOption",
         set = "SetDefineAssistOption",
      },
   },
}


-- Keybindings
BINDING_HEADER_DAHASSISTHELPER = L["Der Assist Helper"]
-- BINDING_NAME_DAHASSIST = "Assist Main Assist"
-- Still needed for binding via the console.
BINDING_NAME_DAHSET = L["Set Main Assist"]

DAH_MACRO = L["Der Assist Helper"]
DAH_ICON = 1


--(BEGIN setup)-----

function DAH:OnInitialize ()
   -- CLI registration.
   LibStub ("AceConfig-3.0"):RegisterOptionsTable ("DerAssistHelper",
                                                   options,
                                                   L["SLASH_COMMANDS"])
end

function DAH:OnEnable ()
end

function DAH:OnDisable ()
end

function DAH:SetAssist ()
   local rc = true

   -- Check combat status first since it cannot
   -- be changed while in combat.
   if InCombatLockdown () then
      self:Print (L["Cannot change assist while in combat."])
      rc = false
   else
      -- Get the target of the player.  If one doesn't exist,
      -- then the macro will just turn into /assist.  
      local name = UnitName ("target")
      if not name or name == UNKNOWNOBJECT or name == UnitName ("player") then
         -- Will result in an unset assist.
         name = nil
      elseif not UnitIsFriend ("player", "target") then
         self:Print (L["You can only assist friendly targets."])
         rc = false
      end

      if rc then
         rc = self:ModifyMacro (name)
      end

      if rc then
         if name then
            self:Print (string.format (L["Assist set to %s."], name))
         else
            self:Print (L["Assist now unset."])
         end
      end
    end

    return rc
end

--(BEGIN option handing)-----

function DAH:GetAssistOption (info)
   return GetBindingKey ("MACRO " .. DAH_MACRO)
end

function DAH:SetAssistOption (info, newValue)
   self:SetKeybind (newValue, "MACRO " .. DAH_MACRO)
   self:Print (string.format (L["Assist macro now bound to..."], newValue))
end

function DAH:GetDefineAssistOption (info)
   return GetBindingKey ("DAHSET")
end

function DAH:SetDefineAssistOption (info, newValue)
   self:SetKeybind (newValue, "DAHSET")
   self:Print (string.format (L["Set assist now bound to..."], newValue))
end

--(BEGIN assist)-----

function DAH:ModifyMacro (target)
   local rc = true

   -- Check combat status first since it cannot
   -- be changed while in combat.
   if InCombatLockdown () then
      self:Print (L["Cannot modify macro while in combat."])
      rc = false
   else
      local macro = L["/assist"]

      if target ~= nil then macro = macro .. " " .. target end

      -- Locate the macro.
      local macroId = GetMacroIndexByName (DAH_MACRO)

      -- If it doesn't exist, try to create it.
      if macroId == 0 then
         -- Make sure there's room.  The DAH macro is
         -- stored within the global macro space.
         local numGlobal = GetNumMacros ()
         if numGlobal < 18 then
            macroId = CreateMacro (DAH_MACRO, DAH_ICON, macro, 0)
         end
      else
         macroid = EditMacro (macroId, DAH_MACRO, DAH_ICON, macro, 0)
      end

      if macroId == 0 then
         self:Print (string.format (L["Unable to create..."], DAH_MACRO))
         rc = false
      end
   end

   return rc
end

function DAH:SetKeybind (keybind, action)
   local rc = true
   
   if InCombatLockdown () then
      self:Print (L["Cannot bind keys while in combat."])
      rc = false
   else
      -- Wipe out all the old bindings.
      while GetBindingKey (action) do
         SetBinding (GetBindingKey (action))
      end
      
      -- Setup the keybind.
      if keybind ~= nil then
         SetBinding (keybind, action)
      end

      -- Flush the settings out to disk.
      SaveBindings (GetCurrentBindingSet ())
   end

   return rc
end
